/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.crop.storage;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Set;
import nl.cloudfarming.client.crop.model.CropBean;

/**
 * Add an instance of this class as PropertyChangeListener to CropBeans to track
 * which CropBeans have changed properties and be able to undo property changes.
 *
 * @author Wytse Visser
 */
public class PropertyChangeTracker implements PropertyChangeListener {

    private OriginalCropBeanProperties properties;

    public PropertyChangeTracker() {
        reset();
    }

    public final void reset() {
        properties = new OriginalCropBeanProperties();
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        OriginalProperties originalProperties = properties.get((CropBean) evt.getSource());
        originalProperties.propertyChange(evt);
    }

    public Set<CropBean> getUpdatedCropBeans() {
        Set<CropBean> updatedBeans = properties.keySet();
        for (CropBean cropBean : updatedBeans) {
            if (properties.get(cropBean).isEmpty()) {
                properties.remove(cropBean);
            }
        }
        return properties.keySet();
    }

    public void undoPropertyChanges() {
        for (CropBean cropBean : getUpdatedCropBeans()) {
            undoNameChange(cropBean);
            undoSourceChange(cropBean);
            undoColorChange(cropBean);
        }
        properties = new OriginalCropBeanProperties();
    }

    private void undoNameChange(CropBean cropBean) {
        if (properties.get(cropBean).containsKey(CropBean.PROP_NAME)) {
            cropBean.setName((String) properties.get(cropBean).get(CropBean.PROP_NAME));
        }
    }

    private void undoSourceChange(CropBean cropBean) {
        if (properties.get(cropBean).containsKey(CropBean.PROP_SOURCE)) {
            cropBean.setSource((String) properties.get(cropBean).get(CropBean.PROP_SOURCE));
        }
    }

    private void undoColorChange(CropBean cropBean) {
        if (properties.get(cropBean).containsKey(CropBean.PROP_COLOR)) {
            cropBean.setColor((Color) properties.get(cropBean).get(CropBean.PROP_COLOR));
        }
    }

    /**
     * This map is used to store the original values for properties of a
     * CropBean. These values can be used to check which CropBeans are changed
     * or to undo these changes.
     */
    private class OriginalCropBeanProperties extends HashMap<CropBean, OriginalProperties> {

        @Override
        public OriginalProperties get(Object key) {
            if (!containsKey((CropBean) key)) {
                put((CropBean) key, new OriginalProperties());
            }
            return super.get(key);
        }
    }

    private class OriginalProperties extends HashMap<String, Object> implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            String propertyName = evt.getPropertyName();
            if (containsKey(propertyName)) {
                removeIfEquals(propertyName, evt.getNewValue());
            } else {
                put(propertyName, evt.getOldValue());
            }
        }

        private void removeIfEquals(String propertyName, Object newValue) {
            Object originalValue = get(propertyName);
            boolean bothNull = originalValue == null && newValue == null;
            if (bothNull || originalValue.equals(newValue)) {
                remove(propertyName);
            }
        }
    }
}
